@@ -7,13 +7,10 @@
 #include <aws/cal/cal.h>
 #include <aws/cal/private/der.h>
 
+#include <CoreFoundation/CoreFoundation.h>
 #include <Security/SecKey.h>
 #include <Security/Security.h>
 
-#if !defined(AWS_OS_IOS)
-#    include <Security/SecSignVerifyTransform.h>
-#endif
-
 struct commoncrypto_ecc_key_pair {
     struct aws_ecc_key_pair key_pair;
     SecKeyRef priv_key_ref;
@@ -25,6 +22,29 @@ static uint8_t s_preamble = 0x04;
 
 static size_t s_der_overhead = 8;
 
+/* The hard-coded "valid" public keys. Copy/pated from one of our unit test. */
+const static uint8_t s_fake_x_ecdsa_p256[] = {
+    0xd0, 0x72, 0x0d, 0xc6, 0x91, 0xaa, 0x80, 0x09, 0x6b, 0xa3, 0x2f, 0xed, 0x1c, 0xb9, 0x7c, 0x2b,
+    0x62, 0x06, 0x90, 0xd0, 0x6d, 0xe0, 0x31, 0x7b, 0x86, 0x18, 0xd5, 0xce, 0x65, 0xeb, 0x72, 0x8f,
+};
+
+const static uint8_t s_fake_y_ecdsa_p256[] = {
+    0x96, 0x81, 0xb5, 0x17, 0xb1, 0xcd, 0xa1, 0x7d, 0x0d, 0x83, 0xd3, 0x35, 0xd9, 0xc4, 0xa8, 0xa9,
+    0xa9, 0xb0, 0xb1, 0xb3, 0xc7, 0x10, 0x6d, 0x8f, 0x3c, 0x72, 0xbc, 0x50, 0x93, 0xdc, 0x27, 0x5f,
+};
+
+const static uint8_t s_fake_x_ecdsa_p384[] = {
+    0xfd, 0x3c, 0x84, 0xe5, 0x68, 0x9b, 0xed, 0x27, 0x0e, 0x60, 0x1b, 0x3d, 0x80, 0xf9, 0x0d, 0x67,
+    0xa9, 0xae, 0x45, 0x1c, 0xce, 0x89, 0x0f, 0x53, 0xe5, 0x83, 0x22, 0x9a, 0xd0, 0xe2, 0xee, 0x64,
+    0x56, 0x11, 0xfa, 0x99, 0x36, 0xdf, 0xa4, 0x53, 0x06, 0xec, 0x18, 0x06, 0x67, 0x74, 0xaa, 0x24,
+};
+
+const static uint8_t s_fake_y_ecdsa_p384[] = {
+    0xb8, 0x3c, 0xa4, 0x12, 0x6c, 0xfc, 0x4c, 0x4d, 0x1d, 0x18, 0xa4, 0xb6, 0xc2, 0x1c, 0x7f, 0x69,
+    0x9d, 0x51, 0x23, 0xdd, 0x9c, 0x24, 0xf6, 0x6f, 0x83, 0x38, 0x46, 0xee, 0xb5, 0x82, 0x96, 0x19,
+    0x6b, 0x42, 0xec, 0x06, 0x42, 0x5d, 0xb5, 0xb7, 0x0a, 0x4b, 0x81, 0xb7, 0xfc, 0xf7, 0x05, 0xa0,
+};
+
 static int s_sign_message(
     const struct aws_ecc_key_pair *key_pair,
     const struct aws_byte_cursor *message,
@@ -211,10 +231,40 @@ struct aws_ecc_key_pair *aws_ecc_key_pair_new_from_private_key_impl(
     enum aws_ecc_curve_name curve_name,
     const struct aws_byte_cursor *priv_key) {
 
-    struct aws_byte_cursor empty_cur;
-    AWS_ZERO_STRUCT(empty_cur);
+    /**
+     * We use SecCreateKeyWithData to create ECC key. Expected format for the key passed to that api is a byte buffer
+     * consisting of "0x04 | x | y | p", where x,y is public pair and p is private key.
+     *
+     * In this case we only have private key and we need to construct SecKey from that.
+     *
+     * We used to just pass 0,0 point for x,y, i.e. "0x04 | 0 | 0 | p".
+     *
+     * This used to work on Macs before 14, but in 14+ SecCreateKeyWithData returns error,
+     * which is reasonable since 0,0 is not a valid public point.
+     *
+     * To get around the issue, we use a fake public key, which is a valid public point, but not matching the private
+     * key as a quick workaround.
+     */
+    struct aws_byte_cursor fake_pub_x;
+    AWS_ZERO_STRUCT(fake_pub_x);
+    struct aws_byte_cursor fake_pub_y;
+    AWS_ZERO_STRUCT(fake_pub_y);
+    switch (curve_name) {
+        case AWS_CAL_ECDSA_P256:
+            fake_pub_x = aws_byte_cursor_from_array(s_fake_x_ecdsa_p256, AWS_ARRAY_SIZE(s_fake_x_ecdsa_p256));
+            fake_pub_y = aws_byte_cursor_from_array(s_fake_y_ecdsa_p256, AWS_ARRAY_SIZE(s_fake_y_ecdsa_p256));
+            break;
+        case AWS_CAL_ECDSA_P384:
+            fake_pub_x = aws_byte_cursor_from_array(s_fake_x_ecdsa_p384, AWS_ARRAY_SIZE(s_fake_x_ecdsa_p384));
+            fake_pub_y = aws_byte_cursor_from_array(s_fake_y_ecdsa_p384, AWS_ARRAY_SIZE(s_fake_y_ecdsa_p384));
+            break;
+        default:
+            aws_raise_error(AWS_ERROR_CAL_UNSUPPORTED_ALGORITHM);
+            return NULL;
+    }
+
     struct commoncrypto_ecc_key_pair *cc_key_pair =
-        s_alloc_pair_and_init_buffers(allocator, curve_name, empty_cur, empty_cur, *priv_key);
+        s_alloc_pair_and_init_buffers(allocator, curve_name, fake_pub_x, fake_pub_y, *priv_key);
 
     if (!cc_key_pair) {
         return NULL;
@@ -252,6 +302,10 @@ struct aws_ecc_key_pair *aws_ecc_key_pair_new_from_private_key_impl(
         goto error;
     }
 
+    /* Zero out the fake public keys in the key pair */
+    aws_byte_buf_secure_zero(&cc_key_pair->key_pair.pub_x);
+    aws_byte_buf_secure_zero(&cc_key_pair->key_pair.pub_y);
+
     CFRelease(key_attributes);
     CFRelease(private_key_data);
 
@@ -334,8 +388,7 @@ error:
     return NULL;
 }
 
-#if !defined(AWS_OS_IOS)
-struct aws_ecc_key_pair *aws_ecc_key_pair_new_generate_random(
+struct aws_ecc_key_pair *aws_ecc_key_pair_new_generate_random_impl(
     struct aws_allocator *allocator,
     enum aws_ecc_curve_name curve_name) {
     struct commoncrypto_ecc_key_pair *cc_key_pair =
@@ -345,10 +398,10 @@ struct aws_ecc_key_pair *aws_ecc_key_pair_new_generate_random(
         return NULL;
     }
 
-    CFDataRef sec_key_export_data = NULL;
     CFStringRef key_size_cf_str = NULL;
     CFMutableDictionaryRef key_attributes = NULL;
-    struct aws_der_decoder *decoder = NULL;
+    CFErrorRef error = NULL;
+    CFDataRef keyData = NULL;
 
     aws_atomic_init_int(&cc_key_pair->key_pair.ref_count, 1);
     cc_key_pair->key_pair.impl = cc_key_pair;
@@ -385,66 +438,41 @@ struct aws_ecc_key_pair *aws_ecc_key_pair_new_generate_random(
 
     CFDictionaryAddValue(key_attributes, kSecAttrKeySizeInBits, key_size_cf_str);
 
-    CFErrorRef error = NULL;
-
     cc_key_pair->priv_key_ref = SecKeyCreateRandomKey(key_attributes, &error);
 
     if (error) {
         aws_raise_error(AWS_ERROR_SYS_CALL_FAILURE);
-        CFRelease(error);
         goto error;
     }
 
     cc_key_pair->pub_key_ref = SecKeyCopyPublicKey(cc_key_pair->priv_key_ref);
 
-    /* OKAY up to here was incredibly reasonable, after this we get attacked by the bad API design
-     * dragons.
-     *
-     * Summary: Apple assumed we'd never need the raw key data. Apple was wrong. So we have to export each component
-     * into the OpenSSL format (just fancy words for DER), but the public key and private key are exported separately
-     * for some reason. Anyways, we export the keys, use our handy dandy DER decoder and grab the raw key data out. */
-    OSStatus ret_code = SecItemExport(cc_key_pair->priv_key_ref, kSecFormatOpenSSL, 0, NULL, &sec_key_export_data);
+    /* Get external representation and parse out components.
+     * SecKeyCopyExternalRepresentation should return 0x | x | y | d for private key */
+    keyData = SecKeyCopyExternalRepresentation(cc_key_pair->priv_key_ref, &error);
 
-    if (ret_code != errSecSuccess) {
+    if (!keyData || error) {
+        if (error) {
+            AWS_LOGF_ERROR(
+                AWS_LS_CAL_ECC, "SecKeyCopyExternalRepresentation call failed with %ld", CFErrorGetCode(error));
+        }
         aws_raise_error(AWS_ERROR_SYS_CALL_FAILURE);
         goto error;
     }
 
-    /* now we need to DER decode data */
-    struct aws_byte_cursor key_cur =
-        aws_byte_cursor_from_array(CFDataGetBytePtr(sec_key_export_data), CFDataGetLength(sec_key_export_data));
-
-    decoder = aws_der_decoder_new(allocator, key_cur);
-
-    if (!decoder) {
-        goto error;
-    }
-
-    struct aws_byte_cursor pub_x;
-    AWS_ZERO_STRUCT(pub_x);
-    struct aws_byte_cursor pub_y;
-    AWS_ZERO_STRUCT(pub_y);
-    struct aws_byte_cursor priv_d;
-    AWS_ZERO_STRUCT(priv_d);
-
-    if (aws_der_decoder_load_ecc_key_pair(decoder, &pub_x, &pub_y, &priv_d, &curve_name)) {
-        goto error;
-    }
-
-    AWS_ASSERT(
-        priv_d.len == key_coordinate_size && pub_x.len == key_coordinate_size && pub_y.len == key_coordinate_size &&
-        "Apple Security Framework had better have exported the full pair.");
+    struct aws_byte_cursor key_data_cur =
+        aws_byte_cursor_from_array(CFDataGetBytePtr(keyData), CFDataGetLength(keyData));
 
     size_t total_buffer_size = key_coordinate_size * 3 + 1;
-
-    if (aws_byte_buf_init(&cc_key_pair->key_pair.key_buf, allocator, total_buffer_size)) {
+    if (key_data_cur.len != total_buffer_size) {
+        AWS_LOGF_ERROR(AWS_LS_CAL_ECC, "Something went wrong. Apple Security Framework didn't return full key data.");
+        aws_raise_error(AWS_ERROR_SYS_CALL_FAILURE);
         goto error;
     }
 
-    aws_byte_buf_write_u8(&cc_key_pair->key_pair.key_buf, s_preamble);
-    aws_byte_buf_append(&cc_key_pair->key_pair.key_buf, &pub_x);
-    aws_byte_buf_append(&cc_key_pair->key_pair.key_buf, &pub_y);
-    aws_byte_buf_append(&cc_key_pair->key_pair.key_buf, &priv_d);
+    if (aws_byte_buf_init_copy_from_cursor(&cc_key_pair->key_pair.key_buf, allocator, key_data_cur)) {
+        goto error;
+    }
 
     /* cc_key_pair->key_pair.key_buf is contiguous memory, so just load up the offsets. */
     cc_key_pair->key_pair.pub_x =
@@ -457,36 +485,34 @@ struct aws_ecc_key_pair *aws_ecc_key_pair_new_generate_random(
     cc_key_pair->key_pair.curve_name = curve_name;
     cc_key_pair->key_pair.vtable = &s_key_pair_vtable;
 
-    CFRelease(sec_key_export_data);
+    CFRelease(keyData);
     CFRelease(key_size_cf_str);
     CFRelease(key_attributes);
-    aws_der_decoder_destroy(decoder);
 
     return &cc_key_pair->key_pair;
 
 error:
-    if (decoder) {
-        aws_der_decoder_destroy(decoder);
+    if (keyData) {
+        CFRelease(keyData);
     }
 
     if (key_attributes) {
         CFRelease(key_attributes);
     }
 
-    if (sec_key_export_data) {
-        CFRelease(sec_key_export_data);
-    }
-
     if (key_size_cf_str) {
         CFRelease(key_size_cf_str);
     }
 
+    if (error) {
+        CFRelease(error);
+    }
+
     s_destroy_key(&cc_key_pair->key_pair);
     return NULL;
 }
-#endif /* AWS_OS_IOS */
 
-struct aws_ecc_key_pair *aws_ecc_key_pair_new_from_asn1(
+struct aws_ecc_key_pair *aws_ecc_key_pair_new_from_asn1_impl(
     struct aws_allocator *allocator,
     const struct aws_byte_cursor *encoded_keys) {
 
